SystemOrganization addCategory: #'Mondrian-Normalizers'!

Object subclass: #MONAbstractNormalizer
	instanceVariableNames: 'command'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Normalizers'!

MONAbstractNormalizer subclass: #MOMultipleColorLinearNormalizer
	instanceVariableNames: 'normalizers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Normalizers'!

!MOMultipleColorLinearNormalizer classSide methodsFor: 'instance creation' stamp: 'simon_denier 9/24/2009 16:10'!
valueRange: anArrayOfNumbers colorRange: anArrayOfColors

	^ self new valueRange: anArrayOfNumbers colorRange: anArrayOfColors! !

!MOMultipleColorLinearNormalizer methodsFor: 'private-accessing' stamp: 'simon_denier 9/24/2009 16:08'!
detectNormalizerFor: aValue

	normalizers keysAndValuesDo: [:range :norm |
		(range rangeIncludes: aValue)
			ifTrue: [^ norm] ].
	self error: aValue asString, ' not in declared ranges'! !

!MOMultipleColorLinearNormalizer methodsFor: 'initialize-release' stamp: 'simon_denier 9/24/2009 17:48'!
initialize

	command := #yourself! !

!MOMultipleColorLinearNormalizer methodsFor: 'accessing' stamp: 'simon_denier 9/24/2009 16:09'!
moValue: anEntity

	| value |
	value := self command value: anEntity.
	"detect range for the value, retrieve and call corresponding normalizers"
	^ (self detectNormalizerFor: value) moValue: value! !

!MOMultipleColorLinearNormalizer methodsFor: 'initialize-release' stamp: 'AlexandreBergel 1/7/2011 11:05'!
valueRange: anArrayOfNumbers colorRange: anArrayOfColors
	" {1. 2. 3} {Color red. Color yellow. Color green}"
	
	| size start end |
	normalizers := SmallDictionary new.
	self assert: anArrayOfNumbers size = anArrayOfColors size.
	size := anArrayOfNumbers size.
	1 to: size - 1 do: [ :i |
		start := anArrayOfNumbers at: i.
		end :=  anArrayOfNumbers at: i + 1.
		normalizers
			at: (start to: end)
			put: (MONColorLinearNormalizer
						inContext: {start. end}
						lowColor: (anArrayOfColors at: i)
						highColor: (anArrayOfColors at: i + 1))
	].
! !

!MONAbstractNormalizer methodsFor: 'accessing' stamp: 'simon_denier 9/24/2009 17:47'!
command
	
	^command! !

!MONAbstractNormalizer methodsFor: 'accessing' stamp: 'Alexandre.Bergel 8/30/2009 13:44'!
command: aBlock
	
	command := aBlock! !

!MONAbstractNormalizer methodsFor: 'accessing' stamp: 'tg 10/5/2008 23:35'!
moValue: anEntity
	
	self subclassResponsibility! !

MONAbstractNormalizer subclass: #MONIdentityNormalizer
	instanceVariableNames: 'dictionary colorIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Normalizers'!

MONIdentityNormalizer subclass: #MONExplicitIdentityNormalizer
	instanceVariableNames: 'colors defaultColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Normalizers'!

!MONExplicitIdentityNormalizer classSide methodsFor: 'instance creation' stamp: ' 5/10/08 23:16'!
withCommand: aBlock withColors: aCollection withDefaultColor: aColor
	
	^(self withCommand: aBlock)
		colors: aCollection;
		defaultColor: aColor;
		yourself! !

!MONExplicitIdentityNormalizer methodsFor: 'accessing' stamp: ' 5/10/08 23:16'!
colors
	
	^colors! !

!MONExplicitIdentityNormalizer methodsFor: 'accessing' stamp: ' 5/10/08 23:16'!
colors: anObject
	
	colors := anObject! !

!MONExplicitIdentityNormalizer methodsFor: 'accessing' stamp: ' 5/10/08 23:16'!
defaultColor
	
	^defaultColor! !

!MONExplicitIdentityNormalizer methodsFor: 'accessing' stamp: ' 5/10/08 23:16'!
defaultColor: anObject
	
	defaultColor := anObject! !

!MONExplicitIdentityNormalizer methodsFor: 'private' stamp: ' 5/10/08 23:16'!
nextColor
	
	colorIndex := colorIndex + 1.
	^colorIndex > self colors size
		ifTrue: [self defaultColor asColor]
		ifFalse: [(self colors at: colorIndex) asColor]! !

!MONIdentityNormalizer classSide methodsFor: 'instance creation' stamp: ' 5/10/08 23:16'!
withCommand: aBlock
	
	^(self new)
		command: aBlock;
		yourself! !

!MONIdentityNormalizer methodsFor: 'initialize-release' stamp: ' 5/10/08 23:16'!
initialize
	
	super initialize.
	colorIndex := 0.
	dictionary := IdentityDictionary new.
	command := #yourself! !

!MONIdentityNormalizer methodsFor: 'accessing' stamp: 'tg 10/5/2008 23:35'!
moValue: anEntity
	
	^dictionary
		at: (command moValue: anEntity)
		ifAbsentPut: [self nextColor]! !

!MONIdentityNormalizer methodsFor: 'private' stamp: 'Alexandre.Bergel 2/22/2009 20:37'!
nextColor
	
	colorIndex := colorIndex \\ Color colorNames size + 1.
	^Color perform: (Color colorNames at: colorIndex)! !

MONAbstractNormalizer subclass: #MONLinearNormalizer
	instanceVariableNames: 'maximumValue context minimumValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Normalizers'!

MONLinearNormalizer subclass: #MONColorLinearNormalizer
	instanceVariableNames: 'minBrightness maxBrightness lowColor highColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Normalizers'!

!MONColorLinearNormalizer classSide methodsFor: 'instance creation' stamp: ' 5/10/08 23:16'!
inContext: aCollection lowColor: lowColor highColor: highColor
	
	| normalizer |
	normalizer := self inContext: aCollection.
	normalizer lowColor: lowColor.
	normalizer highColor: highColor.
	^normalizer! !

!MONColorLinearNormalizer classSide methodsFor: 'instance creation' stamp: ' 5/10/08 23:16'!
inContext: aCollection lowColor: lowColor lowThreshold: lowThreshold highColor: highColor highThreshold: highThreshold
	
	| normalizer |
	normalizer := self inContext: aCollection.
	normalizer lowColor: lowColor.
	normalizer highColor: highColor.
	normalizer minBrightness: lowThreshold.
	normalizer maxBrightness: highThreshold.
	^normalizer! !

!MONColorLinearNormalizer classSide methodsFor: 'instance creation' stamp: ' 5/10/08 23:16'!
inContext: aCollection withCommand: aBlock lowColor: lowColor highColor: highColor
	
	| normalizer |
	normalizer := self inContext: aCollection withCommand: aBlock.
	normalizer lowColor: lowColor.
	normalizer highColor: highColor.
	^normalizer! !

!MONColorLinearNormalizer classSide methodsFor: 'instance creation' stamp: ' 5/10/08 23:16'!
inContext: aCollection withCommand: aBlock lowColor: lowColor lowThreshold: lowThreshold highColor: highColor highThreshold: highThreshold
	
	| normalizer |
	normalizer := self inContext: aCollection withCommand: aBlock.
	normalizer lowColor: lowColor.
	normalizer highColor: highColor.
	normalizer minBrightness: lowThreshold.
	normalizer maxBrightness: highThreshold.
	^normalizer! !

!MONColorLinearNormalizer methodsFor: 'private' stamp: 'tg 10/5/2008 23:51'!
blend: aColor with: aColorValue weight: wt
	"Answer a new ColorValue which is a weighted blend of the receiver and the
	supplied ColorValue."
	"Simply interpolates in RGB space."

	| rv gv bv invWt |
	
	invWt := 1-wt.
	rv := ((aColor red * invWt) + ((aColorValue red)*wt)) "rounded".
	gv := ((aColor green  * invWt) + ((aColorValue green)*wt)) "rounded".
	bv := ((aColor blue *invWt)+ ((aColorValue blue)*wt)) "rounded".
	^Color r: rv g: gv b: bv! !

!MONColorLinearNormalizer methodsFor: 'accessing' stamp: ' 5/10/08 23:16'!
highColor: aBlock
	
	highColor := aBlock! !

!MONColorLinearNormalizer methodsFor: 'translator protocol' stamp: 'tg 10/5/2008 23:32'!
highColorFor: anEntity
	
	^highColor moValue: anEntity! !

!MONColorLinearNormalizer methodsFor: 'initialize-release' stamp: ' 5/10/08 23:16'!
initialize
	
	super initialize.
	lowColor := Color white.
	highColor := Color black! !

!MONColorLinearNormalizer methodsFor: 'accessing' stamp: ' 5/10/08 23:16'!
lowColor: aBlock
	
	lowColor := aBlock! !

!MONColorLinearNormalizer methodsFor: 'translator protocol' stamp: 'tg 10/5/2008 23:32'!
lowColorFor: anEntity
	
	^lowColor moValue: anEntity! !

!MONColorLinearNormalizer methodsFor: 'accessing' stamp: ' 5/10/08 23:16'!
maxBrightness: aBlock
	
	maxBrightness := aBlock! !

!MONColorLinearNormalizer methodsFor: 'translator protocol' stamp: 'tg 10/5/2008 23:32'!
maxBrightnessFor: anEntity
	
	^maxBrightness isNil
		ifTrue: [1.0]
		ifFalse: [maxBrightness moValue: anEntity]! !

!MONColorLinearNormalizer methodsFor: 'accessing' stamp: ' 5/10/08 23:16'!
minBrightness: aBlock
	
	minBrightness := aBlock! !

!MONColorLinearNormalizer methodsFor: 'translator protocol' stamp: 'tg 10/5/2008 23:32'!
minBrightnessFor: anEntity
	
	^minBrightness isNil
		ifTrue: [0.0]
		ifFalse: [minBrightness moValue: anEntity]! !

!MONColorLinearNormalizer methodsFor: 'accessing' stamp: 'TudorGirba 7/12/2011 17:00'!
moValue: anEntity
	"Calculates the color of the entity based on the context."
	
	| myValue myHighColor myLowColor myMinBrightness myMaxBrightness weight minValue maxValue |
	"Get color specific data"
	minValue :=  self minimumValue: anEntity.
	maxValue :=  self maximumValue: anEntity.
	myMinBrightness := self minBrightnessFor: anEntity.
	myMaxBrightness := self maxBrightnessFor: anEntity.
	myHighColor := self highColorFor: anEntity.
	myLowColor := self lowColorFor: anEntity.	"Get data"
	weight := minValue = maxValue 
		ifTrue: [(myMinBrightness + myMaxBrightness) / 2]
		ifFalse: [	
			myValue := (self command value: anEntity) - (self minimumValue: anEntity).
			myMinBrightness + (myValue abs / (maxValue - minValue) * (myMaxBrightness - myMinBrightness)) ].
	^self blend: myLowColor with: myHighColor weight: weight! !

MONLinearNormalizer subclass: #MONFontLinearNormalizer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Normalizers'!

!MONFontLinearNormalizer methodsFor: 'accessing' stamp: 'tg 4/26/2010 22:17'!
moValue: anEntity
	"Calculates the color of the entity based on the context."
	
	| value |
	value := self command moValue: anEntity.
	^5 + (38 * (value abs / (self maximumValue: anEntity))) asInteger! !

!MONLinearNormalizer classSide methodsFor: 'instance creation' stamp: ' 5/10/08 23:16'!
inContext: aCollection
	
	^self inContext: aCollection withCommand: #yourself! !

!MONLinearNormalizer classSide methodsFor: 'instance creation' stamp: ' 5/10/08 23:16'!
inContext: aCollection withCommand: aBlock
	
	^(self new)
		context: aCollection;
		command: aBlock;
		yourself! !

!MONLinearNormalizer methodsFor: 'accessing' stamp: 'Alexandre.Bergel 9/22/2009 17:19'!
command: aBlock
	
	super command: aBlock.
	self flushCache! !

!MONLinearNormalizer methodsFor: 'accessing' stamp: ' 5/10/08 23:16'!
context
	
	^context! !

!MONLinearNormalizer methodsFor: 'accessing' stamp: ' 5/10/08 23:16'!
context: aContext
	
	context := aContext.
	self flushCache! !

!MONLinearNormalizer methodsFor: 'accessing' stamp: ' 5/10/08 23:16'!
flushCache
	
	maximumValue := nil! !

!MONLinearNormalizer methodsFor: 'initialize-release' stamp: ' 5/10/08 23:16'!
initialize
	"Initialize a newly created instance. This method must answer the receiver."
	
	super initialize.
	maximumValue := nil! !

!MONLinearNormalizer methodsFor: 'private' stamp: 'tg 10/5/2008 23:33'!
maximumValue: anEntity
	"Returns the largest value that the color metric provides"
	
	maximumValue isNil
		ifTrue:
			[maximumValue := (self context moValue: anEntity)
				inject: 1
				into:
					[:maximum :entity | maximum max: (self command moValue: entity)]].
	^maximumValue! !

!MONLinearNormalizer methodsFor: 'private' stamp: 'simon.denier 1/7/2011 15:50'!
minimumValue: anEntity
	"Returns the smallest value that the color metric provides"
	
	minimumValue isNil
		ifTrue:
			[minimumValue := (self context moValue: anEntity)
				inject: Float infinity 
				into:
					[:minimum :entity | minimum min: (self command moValue: entity)]].
	^minimumValue! !

MONLinearNormalizer subclass: #MONumberLinearNormalizer
	instanceVariableNames: 'scale'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mondrian-Normalizers'!

!MONumberLinearNormalizer methodsFor: 'initialization' stamp: 'Alexandre.Bergel 8/30/2009 13:53'!
initialize
	super initialize.
	scale := 1! !

!MONumberLinearNormalizer methodsFor: 'accessing' stamp: 'simon.denier 5/15/2009 00:35'!
moValue: anEntity
	"Normalize value based on maximum value."
	
	| value |
	value := self command moValue: anEntity.
	^ ((value abs / (self maximumValue: anEntity)) asFloat * self scale) asInteger! !

!MONumberLinearNormalizer methodsFor: 'accessing' stamp: 'simon.denier 5/15/2009 00:35'!
scale
	^ scale! !

!MONumberLinearNormalizer methodsFor: 'accessing' stamp: 'simon.denier 5/15/2009 00:35'!
scale: anObject
	scale := anObject! !
